PROGRAM HOW_TO_OPEN_LOTS_OF_FILES_IN_A_TURBO_PASCAL_PROGRAM;

{This program demonstrates how to have more than 15 files open simultaneously
 in a single Turbo Pascal program.  No matter how large you set FILES=xxx in
 CONFIG.SYS, you will not normally be able to have more than about 15 files
 open at once; you will get "Runtime error 004 at 0000:0000.".  Apparently,
 TP reserves a stack of only 15 file handles, which under normal circumstances
 is reasonable, but for heavy-duty applications, this is inadequate.  I have
 tested this program under MS-DOS 3.30, and it works perfectly.  Make sure
 that your FILES=xxx statement is at least 5 greater than the maximum number
 of files you intend to have open, or you will end up getting Runtime error 004
 again.  DOS uses a few handles for itself, plus TP apparently uses handles for
 INPUT & OUTPUT, the invisible but ever-present screen & keyboard I/O.}

{Compiled with Turbo Pascal 5.0 & tested under MS-DOS 3.30.  Should work with
 any PC version of Turbo Pascal under DOS 3.0 or greater.}

{90/02/26 - write something to each file rather than just open & close - DWN}
{90/02/26 - source reformatted & opening comments added by David W. Nixon}
{90/02/21 - program written by Christine Sherman of Borland International
              in response to a technical support request from David W. Nixon}

{$M 16384,0,100000} {sets the maximum heap down for new handles}

USES
  DOS;

CONST
  MAXOPEN = 60; {must be at least 5 less than # specified in FILES= statement}

VAR
  I : INTEGER;
  F : ARRAY [1..MAXOPEN] OF TEXT; {i.e. text files}
  NUM : STRING;

PROCEDURE INCREASEHANDLES (HANDLES : BYTE);
{Creates a file handle table with specified number of handles.}
VAR
  R : REGISTERS;
BEGIN
  R.AH := 103;
  R.BX := HANDLES;
  MSDOS (R);
END;

{******** MAINLINE ********}

BEGIN
  INCREASEHANDLES (MAXOPEN+5); {reserve user handles & 5 for DOS}
  {open all files simultaneously}
  FOR I := 1 TO MAXOPEN DO BEGIN
    STR (I,NUM);
    ASSIGN (F[I],'TEST'+NUM+'.BAK');
    REWRITE (F[I]);
    WRITELN ('Opening file #',I);
  END;
  {Write something unique in each file to later verify that no output was
   lost or jumbled.  Conceivably, output intended for file X could end
   up in file X MOD 16, or output to files > 16 could simply disappear.}
  FOR I := 1 to MAXOPEN DO BEGIN
    WRITELN ('Writing to file #',I);
    WRITELN (F[I],'This is File #',I,' of ',MAXOPEN);
  END;
  {close all files}
  FOR I := MAXOPEN DOWNTO 1 DO BEGIN
    CLOSE (F[I]);
    WRITELN ('Closing file #',I);
  END;
END.
